@@ -15,12 +15,17 @@ import (
 	"context"
 	"flag"
 	"fmt"
+	"os"
+	"runtime"
+
+	image_puller_api "github.com/che-incubator/kubernetes-image-puller-operator/pkg/apis"
 	"github.com/eclipse/che-operator/pkg/util"
+	operatorsv1 "github.com/operator-framework/api/pkg/operators/v1"
+	operatorsv1alpha1 "github.com/operator-framework/api/pkg/operators/v1alpha1"
+	packagesv1 "github.com/operator-framework/operator-lifecycle-manager/pkg/package-server/apis/operators/v1"
 	"github.com/operator-framework/operator-sdk/pkg/k8sutil"
 	"github.com/prometheus/common/log"
 	"github.com/sirupsen/logrus"
-	"os"
-	"runtime"
 
 	"github.com/eclipse/che-operator/pkg/apis"
 	"github.com/eclipse/che-operator/pkg/controller"
@@ -31,8 +36,7 @@ import (
 	_ "k8s.io/client-go/plugin/pkg/client/auth/gcp"
 	"sigs.k8s.io/controller-runtime/pkg/client/config"
 	"sigs.k8s.io/controller-runtime/pkg/manager"
-	"sigs.k8s.io/controller-runtime/pkg/runtime/signals"
-	//logf "sigs.k8s.io/controller-runtime/pkg/runtime/log"
+	"sigs.k8s.io/controller-runtime/pkg/manager/signals"
 )
 
 var (
@@ -111,7 +115,10 @@ func main() {
 	defer r.Unset()
 
 	// Create a new Cmd to provide shared dependencies and start components
-	mgr, err := manager.New(cfg, manager.Options{Namespace: namespace})
+	options := manager.Options{
+		Namespace: namespace,
+	}
+	mgr, err := manager.New(cfg, options)
 	if err != nil {
 		log.Error(err, "")
 		os.Exit(1)
@@ -125,6 +132,26 @@ func main() {
 		os.Exit(1)
 	}
 
+	if err := image_puller_api.AddToScheme(mgr.GetScheme()); err != nil {
+		logrus.Error(err, "")
+		os.Exit(1)
+	}
+
+	if err := packagesv1.AddToScheme(mgr.GetScheme()); err != nil {
+		logrus.Error(err, "")
+		os.Exit(1)
+	}
+
+	if err := operatorsv1alpha1.AddToScheme(mgr.GetScheme()); err != nil {
+		log.Error(err, "")
+		os.Exit(1)
+	}
+
+	if err := operatorsv1.AddToScheme(mgr.GetScheme()); err != nil {
+		log.Error(err, "")
+		os.Exit(1)
+	}
+
 	// Setup all Controllers
 	if err := controller.AddToManager(mgr); err != nil {
 		log.Error(err, "")
