@@ -19,6 +19,7 @@ package v1
 // IMPORTANT These 2 last steps are important to ensure backward compatibility with already existing `CheCluster` CRs that were created when no schema was provided.
 
 import (
+	chev1alpha1 "github.com/che-incubator/kubernetes-image-puller-operator/pkg/apis/che/v1alpha1"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/labels"
@@ -50,6 +51,9 @@ type CheClusterSpec struct {
 	// Configuration settings specific to Che installations made on upstream Kubernetes.
 	// +optional
 	K8s CheClusterSpecK8SOnly `json:"k8s"`
+	// Kubernetes Image Puller configuration
+	// +optional
+	ImagePuller CheClusterSpecImagePuller `json:"imagePuller"`
 }
 
 // +k8s:openapi-gen=true
@@ -99,6 +103,10 @@ type CheClusterSpecServer struct {
 	// Enables the debug mode for Che server. Defaults to `false`.
 	// +optional
 	CheDebug string `json:"cheDebug,omitempty"`
+	// Comma-separated list of ClusterRoles that will be assigned to che ServiceAccount.
+	// Be aware that che-operator has to already have all permissions in these ClusterRoles to be able to grant them.
+	// +optional
+	CheClusterRoles string `json:"cheClusterRoles,omitempty"`
 	// Custom cluster role bound to the user for the Che workspaces.
 	// The default roles are used if this is omitted or left blank.
 	// +optional
@@ -137,7 +145,16 @@ type CheClusterSpecServer struct {
 	// This is enabled by default.
 	// Disabling TLS may cause malfunction of some Che components.
 	// +optional
+	// +operator-sdk:gen-csv:customresourcedefinitions.specDescriptors=true
+	// +operator-sdk:gen-csv:customresourcedefinitions.specDescriptors.displayName="Tls support"
+	// +operator-sdk:gen-csv:customresourcedefinitions.specDescriptors.x-descriptors="urn:alm:descriptor:com.tectonic.ui:booleanSwitch"
 	TlsSupport bool `json:"tlsSupport"`
+
+	// Use internal cluster svc names to communicate between components to speed up the traffic
+	// and avoid proxy issues.
+	// The default value is `true`.
+	UseInternalClusterSVCNames bool `json:"useInternalClusterSVCNames"`
+
 	// Public URL of the Devfile registry, that serves sample, ready-to-use devfiles.
 	// You should set it ONLY if you use an external devfile registry (see the `externalDevfileRegistry` field).
 	// By default this will be automatically calculated by the operator.
@@ -388,12 +405,16 @@ type CheClusterSpecAuth struct {
 	// Forces the default `admin` Che user to update password on first login. Defaults to `false`.
 	// +optional
 	UpdateAdminPassword bool `json:"updateAdminPassword"`
-	// Enables the integration of the identity provider (Keycloak / RHSSO) with OpenShift OAuth. Enabled by default on OpenShift.
-	// This will allow users to directly login with their Openshift user through the Openshift login,
+	// Enables the integration of the identity provider (Keycloak / RHSSO) with OpenShift OAuth.
+	// Empty value on the OpenShift platform by default.
+	// If user changes this empty value to true/false, then che-operator respect this value.
+	// Otherwise che-operator tries to auto detect if Openshift oAuth can be enabled and change empty value,
+	// correspondly to auto-detection result.
+	// This property allows users to directly login with their Openshift user through the Openshift login,
 	// and have their workspaces created under personal OpenShift namespaces.
 	// WARNING: the `kubeadmin` user is NOT supported, and logging through it will NOT allow accessing the Che Dashboard.
 	// +optional
-	OpenShiftoAuth bool `json:"openShiftoAuth"`
+	OpenShiftoAuth *bool `json:"openShiftoAuth,omitempty"`
 	// Name of the OpenShift `OAuthClient` resource used to setup identity federation on the OpenShift side. Auto-generated if left blank.
 	// See also the `OpenShiftoAuth` field.
 	// +optional
@@ -479,8 +500,10 @@ type CheClusterSpecK8SOnly struct {
 	// NB: This drives the `is kubernetes.io/ingress.class` annotation on Che-related ingresses.
 	// +optional
 	IngressClass string `json:"ingressClass,omitempty"`
-	// Name of a secret that will be used to setup ingress TLS termination if TLS is enabled.
-	// See also the `tlsSupport` field.
+	// Name of a secret that is used to set ingress TLS termination if TLS is enabled. If the specified secret does not exist,
+	// a self-signed certificate will be created. If the value is empty or omitted, the default ingress controller certificate
+	// will be used. See also the `tlsSupport` field. Note, when switching to the default ingress controller certificate,
+	// `self-signed-certificate` secret should be deleted manually.
 	// +optional
 	TlsSecretName string `json:"tlsSecretName,omitempty"`
 	// FSGroup the Che pod and Workspace pods containers should run in. Defaults to `1724`.
@@ -505,6 +528,23 @@ type CheClusterSpecMetrics struct {
 	Enable bool `json:"enable"`
 }
 
+// +k8s:openapi-gen=true
+// Configuration settings for installation and configuration of the Kubernetes Image Puller
+// See https://github.com/che-incubator/kubernetes-image-puller-operator
+type CheClusterSpecImagePuller struct {
+	// Install and configure the Community Supported Kubernetes Image Puller Operator. If true and no spec is provided,
+	// it will create a default KubernetesImagePuller object to be managed by the Operator.
+	// If false, the KubernetesImagePuller object will be deleted, and the operator will be uninstalled,
+	// regardless of whether or not a spec is provided.
+	//
+	// Please note that while this operator and its behavior is community-supported, its payload may be commercially-supported
+	// if you use it for pulling commercially-supported images.
+	Enable bool `json:"enable"`
+	// A KubernetesImagePullerSpec to configure the image puller in the CheCluster
+	// +optional
+	Spec chev1alpha1.KubernetesImagePullerSpec `json:"spec"`
+}
+
 // CheClusterStatus defines the observed state of Che installation
 type CheClusterStatus struct {
 	// Indicates if or not a Postgres instance has been correctly provisioned
@@ -518,15 +558,27 @@ type CheClusterStatus struct {
 	OpenShiftoAuthProvisioned bool `json:"openShiftoAuthProvisioned"`
 	// Status of a Che installation. Can be `Available`, `Unavailable`, or `Available, Rolling Update in Progress`
 	// +optional
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors=true
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.displayName="Status"
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.x-descriptors="urn:alm:descriptor:io.kubernetes.phase"
 	CheClusterRunning string `json:"cheClusterRunning"`
 	// Current installed Che version
 	// +optional
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors=true
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.displayName="displayName: Eclipse Che version"
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.x-descriptors="urn:alm:descriptor:org.w3:link"
 	CheVersion string `json:"cheVersion"`
 	// Public URL to the Che server
 	// +optional
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors=true
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.displayName="Eclipse Che URL"
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.x-descriptors="urn:alm:descriptor:org.w3:link"
 	CheURL string `json:"cheURL"`
 	// Public URL to the Identity Provider server (Keycloak / RH SSO).
 	// +optional
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors=true
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.displayName="Keycloak Admin Console URL"
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.x-descriptors="urn:alm:descriptor:org.w3:link"
 	KeycloakURL string `json:"keycloakURL"`
 	// Public URL to the Devfile registry
 	// +optional
@@ -536,12 +588,21 @@ type CheClusterStatus struct {
 	PluginRegistryURL string `json:"pluginRegistryURL"`
 	// A human readable message indicating details about why the pod is in this condition.
 	// +optional
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors=true
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.displayName="Message"
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.x-descriptors="urn:alm:descriptor:text"
 	Message string `json:"message,omitempty"`
 	// A brief CamelCase message indicating details about why the pod is in this state.
 	// +optional
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors=true
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.displayName="Reason"
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.x-descriptors="urn:alm:descriptor:text"
 	Reason string `json:"reason,omitempty"`
 	// A URL that can point to some URL where to find help related to the current Operator status.
 	// +optional
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors=true
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.displayName="Help link"
+	// +operator-sdk:gen-csv:customresourcedefinitions.statusDescriptors.x-descriptors="urn:alm:descriptor:org.w3:link"
 	HelpLink string `json:"helpLink,omitempty"`
 }
 
@@ -550,6 +611,7 @@ type CheClusterStatus struct {
 // The `CheCluster` custom resource allows defining and managing a Che server installation
 // +k8s:openapi-gen=true
 // +kubebuilder:subresource:status
+// +operator-sdk:gen-csv:customresourcedefinitions.displayName="Eclipse Che Cluster"
 type CheCluster struct {
 	metav1.TypeMeta   `json:",inline"`
 	metav1.ObjectMeta `json:"metadata,omitempty"`
@@ -582,3 +644,7 @@ func (c *CheCluster) IsAirGapMode() bool {
 	return c.Spec.Server.AirGapContainerRegistryHostname != "" ||
 		c.Spec.Server.AirGapContainerRegistryOrganization != ""
 }
+
+func (c *CheCluster) IsImagePullerSpecEmpty() bool {
+	return c.Spec.ImagePuller.Spec == (chev1alpha1.KubernetesImagePullerSpec{})
+}
