@@ -13,39 +13,56 @@ package che
 
 import (
 	"context"
-	"github.com/eclipse/che-operator/pkg/deploy/server"
 
 	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
 	"github.com/eclipse/che-operator/pkg/deploy"
+	"github.com/eclipse/che-operator/pkg/deploy/server"
 	"github.com/eclipse/che-operator/pkg/util"
 	configv1 "github.com/openshift/api/config/v1"
 	"k8s.io/apimachinery/pkg/types"
 )
 
 func (r *ReconcileChe) getProxyConfiguration(checluster *orgv1.CheCluster) (*deploy.Proxy, error) {
-	proxy, err := deploy.ReadCheClusterProxyConfiguration(checluster)
-	if err != nil {
-		return nil, err
-	}
-
+	// OpenShift 4.x
 	if util.IsOpenShift4 {
 		clusterProxy := &configv1.Proxy{}
 		if err := r.client.Get(context.TODO(), types.NamespacedName{Name: "cluster"}, clusterProxy); err != nil {
 			return nil, err
 		}
 
-		// If proxy configuration exists in CR then cluster wide proxy configuration is ignored
-		// otherwise cluster wide proxy configuration is used and non proxy hosts
-		// are merted with defined ones in CR
-		if proxy.HttpProxy == "" && clusterProxy.Status.HTTPProxy != "" {
-			proxy, err = deploy.ReadClusterWideProxyConfiguration(clusterProxy, proxy.NoProxy)
-			if err != nil {
-				return nil, err
-			}
+		clusterWideProxyConf, err := deploy.ReadClusterWideProxyConfiguration(clusterProxy)
+		if err != nil {
+			return nil, err
 		}
+
+		cheClusterProxyConf, err := deploy.ReadCheClusterProxyConfiguration(checluster)
+		if err != nil {
+			return nil, err
+		}
+
+		// If proxy configuration exists in CR then cluster wide proxy configuration is ignored
+		// Non proxy hosts are merged
+		if cheClusterProxyConf.HttpProxy != "" {
+			cheClusterProxyConf.NoProxy = deploy.MergeNonProxy(cheClusterProxyConf.NoProxy, clusterWideProxyConf.NoProxy)
+			return cheClusterProxyConf, nil
+		} else if clusterWideProxyConf.HttpProxy != "" {
+			clusterWideProxyConf.NoProxy = deploy.MergeNonProxy(clusterWideProxyConf.NoProxy, cheClusterProxyConf.NoProxy)
+			return clusterWideProxyConf, nil
+		}
+
+		// proxy isn't configured
+		return &deploy.Proxy{}, nil
 	}
 
-	return proxy, nil
+	// OpenShift 3.x and k8s
+	cheClusterProxyConf, err := deploy.ReadCheClusterProxyConfiguration(checluster)
+	if err != nil {
+		return nil, err
+	}
+	if checluster.Spec.Server.UseInternalClusterSVCNames {
+		cheClusterProxyConf.NoProxy = deploy.MergeNonProxy(cheClusterProxyConf.NoProxy, ".svc")
+	}
+	return cheClusterProxyConf, nil
 }
 
 func (r *ReconcileChe) putOpenShiftCertsIntoConfigMap(deployContext *deploy.DeployContext) (bool, error) {
