@@ -14,11 +14,12 @@ package deploy
 
 import (
 	"fmt"
-	"gopkg.in/yaml.v2"
 	"io/ioutil"
 	"os"
 	"strings"
 
+	"gopkg.in/yaml.v2"
+
 	"github.com/eclipse/che-operator/pkg/util"
 	"github.com/sirupsen/logrus"
 	appsv1 "k8s.io/api/apps/v1"
@@ -61,6 +62,7 @@ const (
 	DefaultPluginRegistryMemoryLimit   = "256Mi"
 	DefaultPluginRegistryMemoryRequest = "16Mi"
 
+	// DefaultKube
 	DefaultDevfileRegistryMemoryLimit   = "256Mi"
 	DefaultDevfileRegistryMemoryRequest = "16Mi"
 	DefaultKeycloakAdminUserName        = "admin"
@@ -83,6 +85,8 @@ const (
 	DefaultSecurityContextFsGroup   = "1724"
 	DefaultSecurityContextRunAsUser = "1724"
 
+	KubernetesImagePullerOperatorCSV = "kubernetes-imagepuller-operator.v0.0.4"
+
 	DefaultServerExposureStrategy           = "multi-host"
 	DefaultKubernetesSingleHostExposureType = "native"
 	DefaultOpenShiftSingleHostExposureType  = "gateway"
@@ -97,6 +101,17 @@ const (
 	OldDefaultCodeReadyServerImageRepo = "registry.redhat.io/codeready-workspaces/server-rhel8"
 	OldDefaultCodeReadyServerImageTag  = "1.2"
 	OldCrwPluginRegistryUrl            = "https://che-plugin-registry.openshift.io"
+
+	// kubernetes default labels
+	KubernetesComponentLabelKey = "app.kubernetes.io/component"
+	KubernetesPartOfLabelKey    = "app.kubernetes.io/part-of"
+
+	CheEclipseOrg = "che.eclipse.org"
+
+	// che.eclipse.org annotations
+	CheEclipseOrgMountPath = "che.eclipse.org/mount-path"
+	CheEclipseOrgMountAs   = "che.eclipse.org/mount-as"
+	CheEclipseOrgEnvName   = "che.eclipse.org/env-name"
 )
 
 func InitDefaults(defaultsPath string) {
@@ -256,6 +271,10 @@ func DefaultSingleHostGatewayConfigSidecarImage(cr *orgv1.CheCluster) string {
 	return patchDefaultImageName(cr, defaultSingleHostGatewayConfigSidecarImage)
 }
 
+func DefaultKubernetesImagePullerOperatorCSV() string {
+	return KubernetesImagePullerOperatorCSV
+}
+
 func DefaultPullPolicyFromDockerImage(dockerImage string) string {
 	tag := "latest"
 	parts := strings.Split(dockerImage, ":")
@@ -386,6 +405,7 @@ func InitTestDefaultsFromDeployment(deploymentFile string) error {
 		}
 	}
 
+	os.Setenv("MOCK_API", "1")
 	InitDefaultsFromEnv()
 	return nil
 }
