@@ -18,16 +18,22 @@ import (
 	"encoding/pem"
 	stderrors "errors"
 	"net/http"
+	"reflect"
 	"strings"
 	"time"
 
+	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
 	"github.com/eclipse/che-operator/pkg/util"
 	routev1 "github.com/openshift/api/route/v1"
 	"github.com/sirupsen/logrus"
 	batchv1 "k8s.io/api/batch/v1"
 	corev1 "k8s.io/api/core/v1"
+	"k8s.io/api/extensions/v1beta1"
 	"k8s.io/apimachinery/pkg/api/errors"
+	"k8s.io/apimachinery/pkg/labels"
+	"k8s.io/apimachinery/pkg/selection"
 	"k8s.io/apimachinery/pkg/types"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 )
@@ -41,6 +47,21 @@ const (
 	CheTLSJobComponentName                = "che-create-tls-secret-job"
 	CheTLSSelfSignedCertificateSecretName = "self-signed-certificate"
 	DefaultCheTLSSecretName               = "che-tls"
+
+	// CheCACertsConfigMapLabelKey is the label key which marks config map with additional CA certificates
+	CheCACertsConfigMapLabelKey = "app.kubernetes.io/component"
+	// CheCACertsConfigMapLabelKey is the label value which marks config map with additional CA certificates
+	CheCACertsConfigMapLabelValue = "ca-bundle"
+	// CheAllCACertsConfigMapName is the name of config map which contains all additional trusted by Che TLS CA certificates
+	CheAllCACertsConfigMapName = "ca-certs-merged"
+	// CheMergedCAConfigMapRevisionsAnnotationKey is annotation name which holds versions of included config maps in format: cm-name1=ver1,cm-name2=ver2
+	CheMergedCAConfigMapRevisionsAnnotationKey = "che.eclipse.org/included-configmaps"
+
+	// Local constants
+	// labelEqualSign consyant is used as a replacement for '=' symbol in labels because '=' is not allowed there
+	labelEqualSign = "-"
+	// labelCommaSign consyant is used as a replacement for ',' symbol in labels because ',' is not allowed there
+	labelCommaSign = "."
 )
 
 // IsSelfSignedCertificateUsed detects whether endpoints are/should be secured by self-signed certificate.
@@ -60,99 +81,141 @@ func IsSelfSignedCertificateUsed(deployContext *DeployContext) (bool, error) {
 		return false, err
 	}
 
-	if util.IsOpenShift {
-		// Get router TLS certificates chain
-		peerCertificates, err := GetEndpointTLSCrtChain(deployContext, "")
-		if err != nil {
-			return false, err
-		}
+	if !util.IsOpenShift {
+		// Handle custom tls secret for Che ingresses
+		cheTLSSecretName := deployContext.CheCluster.Spec.K8s.TlsSecretName
+		if cheTLSSecretName != "" {
+			// The secret is specified in CR
+			cheTLSSecret := &corev1.Secret{}
+			err = deployContext.ClusterAPI.Client.Get(context.TODO(), types.NamespacedName{Namespace: deployContext.CheCluster.Namespace, Name: cheTLSSecretName}, cheTLSSecret)
+			if err != nil {
+				if !errors.IsNotFound(err) {
+					// Failed to get secret, return error to restart reconcile loop.
+					return false, err
+				}
 
-		// Check the chain if ti contains self-signed CA certificate
-		for _, cert := range peerCertificates {
-			if cert.Subject.String() == cert.Issuer.String() {
-				// Self-signed CA certificate is found in the chain
+				// Both secrets (che-tls and self-signed-certificate) are missing which means that we should generate them (i.e. use self-signed certificate).
 				return true, nil
 			}
+			// TLS secret found, consider it as commonly trusted.
+			return false, nil
 		}
-		// The chain doesn't contain self-signed certificate
-		return false, nil
+
+		// Empty secret name means using of default ingress certificate.
+		// Retrieve the info about certificate chain from test ingress below.
 	}
 
-	// For Kubernetes, check che-tls secret.
-
-	cheTLSSecretName := deployContext.CheCluster.Spec.K8s.TlsSecretName
-	if len(cheTLSSecretName) < 1 {
-		cheTLSSecretName = DefaultCheTLSSecretName
-	}
-
-	cheTLSSecret := &corev1.Secret{}
-	err = deployContext.ClusterAPI.Client.Get(context.TODO(), types.NamespacedName{Namespace: deployContext.CheCluster.Namespace, Name: cheTLSSecretName}, cheTLSSecret)
+	// Get route/ingress TLS certificates chain
+	peerCertificates, err := GetEndpointTLSCrtChain(deployContext, "")
 	if err != nil {
-		if !errors.IsNotFound(err) {
-			// Failed to get secret, return error to restart reconcile loop.
-			return false, err
-		}
-
-		// Both secrets (che-tls and self-signed-certificate) are missing which means that we should generate them (i.e. use self-signed certificate).
-		return true, nil
+		return false, err
 	}
-	// TLS secret found, consider it as commonly trusted.
+
+	// Check the chain if it contains self-signed CA certificate
+	for _, cert := range peerCertificates {
+		if cert.Subject.String() == cert.Issuer.String() {
+			// Self-signed CA certificate is found in the chain
+			return true, nil
+		}
+	}
+	// The chain doesn't contain self-signed certificate
 	return false, nil
 }
 
 // GetEndpointTLSCrtChain retrieves TLS certificates chain from given endpoint.
-// If endpoint is not specified, then a test route will be created and used to get router certificates.
+// If endpoint is not specified, then a test route/ingress will be created and used to get router certificates.
 func GetEndpointTLSCrtChain(deployContext *DeployContext, endpointURL string) ([]*x509.Certificate, error) {
 	if util.IsTestMode() {
 		return nil, stderrors.New("Not allowed for tests")
 	}
 
-	var isTestRoute bool = len(endpointURL) < 1
+	var useTestEndpoint bool = len(endpointURL) < 1
 	var requestURL string
-	if isTestRoute {
-		// Create test route to get certificates chain.
-		// Note, it is not possible to use SyncRouteToCluster here as it may cause infinite reconcile loop.
-		additionalLabels := deployContext.CheCluster.Spec.Server.CheServerRoute.Labels
-		routeSpec, err := GetSpecRoute(deployContext, "test", "", "test", 8080, additionalLabels)
-		if err != nil {
-			return nil, err
-		}
-		// Remove controller reference to prevent queueing new reconcile loop
-		routeSpec.SetOwnerReferences(nil)
-		// Create route manually
-		if err := deployContext.ClusterAPI.Client.Create(context.TODO(), routeSpec); err != nil {
-			if !errors.IsAlreadyExists(err) {
-				logrus.Errorf("Failed to create test route 'test': %s", err)
-				return nil, err
-			}
-		}
-
-		// Schedule test route cleanup after the job done.
-		defer func() {
-			if err := deployContext.ClusterAPI.Client.Delete(context.TODO(), routeSpec); err != nil {
-				logrus.Errorf("Failed to delete test route %s: %s", routeSpec.Name, err)
-			}
-		}()
-
-		// Wait till route is ready
-		var route *routev1.Route
-		for wait := true; wait; {
-			time.Sleep(time.Duration(1) * time.Second)
-			route, err = GetClusterRoute(routeSpec.Name, routeSpec.Namespace, deployContext.ClusterAPI.Client)
+	if useTestEndpoint {
+		if util.IsOpenShift {
+			// Create test route to get certificates chain.
+			// Note, it is not possible to use SyncRouteToCluster here as it may cause infinite reconcile loop.
+			additionalLabels := deployContext.CheCluster.Spec.Server.CheServerRoute.Labels
+			routeSpec, err := GetSpecRoute(deployContext, "test", "", "test", 8080, additionalLabels)
 			if err != nil {
 				return nil, err
 			}
-			wait = len(route.Spec.Host) == 0
-		}
+			// Remove controller reference to prevent queueing new reconcile loop
+			routeSpec.SetOwnerReferences(nil)
+			// Create route manually
+			if err := deployContext.ClusterAPI.Client.Create(context.TODO(), routeSpec); err != nil {
+				if !errors.IsAlreadyExists(err) {
+					logrus.Errorf("Failed to create test route 'test': %s", err)
+					return nil, err
+				}
+			}
 
-		requestURL = "https://" + route.Spec.Host
+			// Schedule test route cleanup after the job done.
+			defer func() {
+				if err := deployContext.ClusterAPI.Client.Delete(context.TODO(), routeSpec); err != nil {
+					logrus.Errorf("Failed to delete test route %s: %s", routeSpec.Name, err)
+				}
+			}()
+
+			// Wait till the route is ready
+			var route *routev1.Route
+			for wait := true; wait; {
+				time.Sleep(time.Duration(1) * time.Second)
+				route, err = GetClusterRoute(routeSpec.Name, routeSpec.Namespace, deployContext.ClusterAPI.Client)
+				if err != nil {
+					return nil, err
+				}
+				wait = len(route.Spec.Host) == 0
+			}
+
+			requestURL = "https://" + route.Spec.Host
+		} else {
+			// Kubernetes
+
+			// Create test ingress to get certificates chain.
+			// Note, it is not possible to use SyncIngressToCluster here as it may cause infinite reconcile loop.
+			additionalLabels := deployContext.CheCluster.Spec.Server.CheServerIngress.Labels
+			ingressSpec, err := GetSpecIngress(deployContext, "test", "", "test", 8080, additionalLabels)
+			if err != nil {
+				return nil, err
+			}
+			// Remove controller reference to prevent queueing new reconcile loop
+			ingressSpec.SetOwnerReferences(nil)
+			// Create ingress manually
+			if err := deployContext.ClusterAPI.Client.Create(context.TODO(), ingressSpec); err != nil {
+				if !errors.IsAlreadyExists(err) {
+					logrus.Errorf("Failed to create test ingress 'test': %s", err)
+					return nil, err
+				}
+			}
+
+			// Schedule test ingress cleanup after the job done.
+			defer func() {
+				if err := deployContext.ClusterAPI.Client.Delete(context.TODO(), ingressSpec); err != nil {
+					logrus.Errorf("Failed to delete test ingress %s: %s", ingressSpec.Name, err)
+				}
+			}()
+
+			// Wait till the ingress is ready
+			var ingress *v1beta1.Ingress
+			for wait := true; wait; {
+				time.Sleep(time.Duration(1) * time.Second)
+				ingress, err = GetClusterIngress(ingressSpec.Name, ingressSpec.Namespace, deployContext.ClusterAPI.Client)
+				if err != nil {
+					return nil, err
+				}
+				wait = len(ingress.Spec.Rules[0].Host) == 0
+			}
+
+			requestURL = "https://" + ingress.Spec.Rules[0].Host
+		}
 	} else {
 		requestURL = endpointURL
 	}
 
-	certificates, err := doRequestForTLSCrtChain(deployContext, requestURL, isTestRoute)
+	certificates, err := doRequestForTLSCrtChain(deployContext, requestURL, useTestEndpoint)
 	if err != nil {
-		if deployContext.Proxy.HttpProxy != "" && isTestRoute {
+		if deployContext.Proxy.HttpProxy != "" && useTestEndpoint {
 			// Fetching certificates from the test route without proxy failed. Probably non-proxy connections are blocked.
 			// Retrying with proxy configuration, however it might cause retreiving of wrong certificate in case of TLS interception by proxy.
 			logrus.Warn("Failed to get certificate chain of trust of the OpenShift Ingress bypassing the proxy")
@@ -188,7 +251,8 @@ func doRequestForTLSCrtChain(deployContext *DeployContext, requestURL string, sk
 	return resp.TLS.PeerCertificates, nil
 }
 
-// GetEndpointTLSCrtBytes creates a test TLS route and gets it to extract certificate chain
+// GetEndpointTLSCrtBytes extracts certificate chain from given endpoint.
+// Creates a test TLS route/ingress if endpoint url is empty.
 // There's an easier way which is to read tls secret in default (3.11) or openshift-ingress (4.0) namespace
 // which however requires extra privileges for operator service account
 func GetEndpointTLSCrtBytes(deployContext *DeployContext, endpointURL string) (certificates []byte, err error) {
@@ -254,7 +318,7 @@ func K8sHandleCheTLSSecrets(deployContext *DeployContext) (reconcile.Result, err
 			return reconcile.Result{RequeueAfter: time.Second}, err
 		}
 
-		role, err := SyncRoleToCluster(deployContext, CheTLSJobRoleName, []string{"secrets"}, []string{"create"})
+		role, err := SyncTLSRoleToCluster(deployContext)
 		if role == nil {
 			return reconcile.Result{RequeueAfter: time.Second}, err
 		}
@@ -382,19 +446,6 @@ func K8sHandleCheTLSSecrets(deployContext *DeployContext) (reconcile.Result, err
 	return reconcile.Result{}, nil
 }
 
-// CheckAndUpdateK8sTLSConfiguration validates Che TLS configuration on Kubernetes infra.
-// If configuration is wrong it will guess most common use cases and will make changes in Che CR accordingly to the assumption.
-func CheckAndUpdateK8sTLSConfiguration(deployContext *DeployContext) error {
-	if deployContext.CheCluster.Spec.K8s.TlsSecretName == "" {
-		deployContext.CheCluster.Spec.K8s.TlsSecretName = DefaultCheTLSSecretName
-		if err := UpdateCheCRSpec(deployContext, "TlsSecretName", deployContext.CheCluster.Spec.K8s.TlsSecretName); err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
 func isCheTLSSecretValid(cheTLSSecret *corev1.Secret) bool {
 	if data, exists := cheTLSSecret.Data["tls.key"]; !exists || len(data) == 0 {
 		return false
@@ -429,3 +480,116 @@ func deleteJob(deployContext *DeployContext, job *batchv1.Job) {
 		logrus.Errorf("Error deleting job: '%s', error: %v", CheTLSJobName, err)
 	}
 }
+
+// SyncAdditionalCACertsConfigMapToCluster makes sure that additional CA certs config map is up to date if any
+func SyncAdditionalCACertsConfigMapToCluster(cr *orgv1.CheCluster, deployContext *DeployContext) (*corev1.ConfigMap, error) {
+	// Get all source config maps, if any
+	caConfigMaps, err := getCACertsConfigMaps(deployContext)
+	if err != nil {
+		return nil, err
+	}
+	if len(cr.Spec.Server.ServerTrustStoreConfigMapName) > 0 {
+		crConfigMap := &corev1.ConfigMap{}
+		err := deployContext.ClusterAPI.Client.Get(context.TODO(), types.NamespacedName{Namespace: deployContext.CheCluster.Namespace, Name: cr.Spec.Server.ServerTrustStoreConfigMapName}, crConfigMap)
+		if err != nil {
+			return nil, err
+		}
+		caConfigMaps = append(caConfigMaps, *crConfigMap)
+	}
+
+	mergedCAConfigMap := &corev1.ConfigMap{}
+	err = deployContext.ClusterAPI.Client.Get(context.TODO(), types.NamespacedName{Namespace: deployContext.CheCluster.Namespace, Name: CheAllCACertsConfigMapName}, mergedCAConfigMap)
+	if err == nil {
+		// Merged config map exists. Check if it is up to date.
+		caConfigMapsCurrentRevisions := make(map[string]string)
+		for _, cm := range caConfigMaps {
+			caConfigMapsCurrentRevisions[cm.Name] = cm.ResourceVersion
+		}
+
+		caConfigMapsCachedRevisions := make(map[string]string)
+		if mergedCAConfigMap.ObjectMeta.Annotations != nil {
+			if revisions, exists := mergedCAConfigMap.ObjectMeta.Annotations[CheMergedCAConfigMapRevisionsAnnotationKey]; exists {
+				for _, cmNameRevision := range strings.Split(revisions, labelCommaSign) {
+					nameRevision := strings.Split(cmNameRevision, labelEqualSign)
+					if len(nameRevision) != 2 {
+						// The label value is invalid, recreate merged config map
+						break
+					}
+					caConfigMapsCachedRevisions[nameRevision[0]] = nameRevision[1]
+				}
+			}
+		}
+
+		if reflect.DeepEqual(caConfigMapsCurrentRevisions, caConfigMapsCachedRevisions) {
+			// Existing merged config map is up to date, do nothing
+			return mergedCAConfigMap, nil
+		}
+	} else {
+		if !errors.IsNotFound(err) {
+			return nil, err
+		}
+		// Merged config map doesn't exist. Create it.
+	}
+
+	// Merged config map is out of date or doesn't exist
+	// Merge all config maps into single one to mount inside Che components and workspaces
+	data := make(map[string]string)
+	revisions := ""
+	for _, cm := range caConfigMaps {
+		// Copy data
+		for key, dataRecord := range cm.Data {
+			data[cm.ObjectMeta.Name+"."+key] = dataRecord
+		}
+
+		// Save source config map revision
+		if revisions != "" {
+			revisions += labelCommaSign
+		}
+		revisions += cm.ObjectMeta.Name + labelEqualSign + cm.ObjectMeta.ResourceVersion
+	}
+
+	mergedCAConfigMapSpec, err := GetSpecConfigMap(deployContext, CheAllCACertsConfigMapName, data)
+	if err != nil {
+		return nil, err
+	}
+	mergedCAConfigMapSpec.ObjectMeta.Labels[KubernetesPartOfLabelKey] = CheEclipseOrg
+
+	if mergedCAConfigMapSpec.ObjectMeta.Annotations == nil {
+		mergedCAConfigMapSpec.ObjectMeta.Annotations = make(map[string]string)
+	}
+	mergedCAConfigMapSpec.ObjectMeta.Annotations[CheMergedCAConfigMapRevisionsAnnotationKey] = revisions
+
+	logrus.Infof("Updating additional CA certs config map: %s", CheAllCACertsConfigMapName)
+	mergedCAConfigMap, err = SyncConfigMapToCluster(deployContext, mergedCAConfigMapSpec)
+	if err != nil {
+		return nil, err
+	}
+	return mergedCAConfigMap, nil
+}
+
+// getCACertsConfigMaps returns list of config maps with additional CA certificates that should be trusted by Che
+// The selection is based on the specific label
+func getCACertsConfigMaps(deployContext *DeployContext) ([]corev1.ConfigMap, error) {
+	CACertsConfigMapList := &corev1.ConfigMapList{}
+
+	caBundleLabelSelectorRequirement, _ := labels.NewRequirement(CheCACertsConfigMapLabelKey, selection.Equals, []string{CheCACertsConfigMapLabelValue})
+	cheComponetLabelSelectorRequirement, _ := labels.NewRequirement(KubernetesPartOfLabelKey, selection.Equals, []string{CheEclipseOrg})
+	listOptions := &client.ListOptions{
+		LabelSelector: labels.NewSelector().Add(*cheComponetLabelSelectorRequirement).Add(*caBundleLabelSelectorRequirement),
+	}
+	if err := deployContext.ClusterAPI.Client.List(context.TODO(), CACertsConfigMapList, listOptions); err != nil {
+		return nil, err
+	}
+
+	return CACertsConfigMapList.Items, nil
+}
+
+// GetAdditionalCACertsConfigMapVersion returns revision of merged additional CA certs config map
+func GetAdditionalCACertsConfigMapVersion(deployContext *DeployContext) string {
+	trustStoreConfigMap, _ := GetClusterConfigMap(CheAllCACertsConfigMapName, deployContext.CheCluster.Namespace, deployContext.ClusterAPI.Client)
+	if trustStoreConfigMap != nil {
+		return trustStoreConfigMap.ResourceVersion
+	}
+
+	return ""
+}
